-- Root.lua

Adventure = Adventure or {}

-- CoroutineUtility
CoroutineUtility = dofile('Common/CoroutineUtility.lua')

-- TransformUtility
local TransformUtility = dofile('Common/TransformUtility.lua')
local TransformUtility_metatable = { __index = TransformUtility }

-- Character
setmetatable(Adventure.Character, TransformUtility_metatable)

-- Camera
setmetatable(Adventure.Camera3D, TransformUtility_metatable)

-- Bg
setmetatable(Adventure.Bg, TransformUtility_metatable)

-- ThreadList
ThreadList = dofile('Common/ThreadList.lua')

-- AdventureBattle
do
	local AdventureBattle = dofile('Adventure/AdventureBattle.lua')
	setmetatable(Adventure.Battle, { __index = AdventureBattle })
end

-- local env
Adventure.local_script_env_ = {}

-- __newindex function
local function newindex(t, key, value)
	error('Can\'t set global table value in script. You tried to set a \''..key..'\'.', 2)
end

-- script env
Adventure.script_env_ = dofile('Adventure/ScriptEnv.lua')
do
	local metatable = {}
	metatable.__index = Adventure.script_env_
	metatable.__newindex = newindex
	Adventure.script_env_metatable_ = metatable
end

-- current script
Adventure.current_script_ = nil

-- functions
function Adventure.SetScript(script, script_env, ...)
	-- last script finalize
	if Adventure.current_script_ ~= nil then
		local current_script_env = Adventure.current_script_.env_
		Adventure.local_script_env_ = current_script_env
		Adventure.script_env_.DisposeAllRegistedObject()
		setmetatable(current_script_env, nil)
		Adventure.local_script_env_ = nil
	end

	-- set
	if script == nil then
		Adventure.current_script_ = nil
	else
		script_env.thread_list_ = ThreadList.New()
		script_env.disposable_object_list_ = {}
		setmetatable(script_env, Adventure.script_env_metatable_)

		local script_coroutine = coroutine.create(Adventure.ScriptCoroutine)
		local resume_result = { coroutine.resume(script_coroutine, script, script_env, ...) }
		if resume_result[1] ~= false then
			Adventure.current_script_ = {
				env_ = script_env,
				coroutine_ = script_coroutine,
			}
		else
			ThreadList.CoroutineError(script_coroutine, resume_result[2])
		end
	end
end

function Adventure.ResumeScript(elapsed_time)
	if Adventure.current_script_ ~= nil then
		local script_coroutine = Adventure.current_script_.coroutine_
		local resume_result = { coroutine.resume(script_coroutine, elapsed_time) }
		if coroutine.status(script_coroutine) == 'dead' then
			Adventure.SetScript(nil)
		end

		if resume_result[1] == false then
			ThreadList.CoroutineError(script_coroutine, resume_result[2])
		end
	end
	return Adventure.current_script_ ~= nil
end

function Adventure.ScriptCoroutine(script, script_env, ...)
	local thread = coroutine.create(script)
	coroutine.yield()

	Adventure.local_script_env_ = script_env

	local resume_result = { coroutine.resume(thread, ...) }
	local elapsed_time = 0.0
	while coroutine.status(thread) ~= 'dead' do
		script_env.thread_list_:Resume(elapsed_time)
		elapsed_time = coroutine.yield()
		Adventure.local_script_env_ = script_env
		resume_result = { coroutine.resume(thread, elapsed_time) }
	end

	if resume_result[1] == false then
		ThreadList.CoroutineError(thread, resume_result[2])
	end
end
