-- ScriptEnv.lua

local function LocalScriptEnv()
	return Adventure.local_script_env_
end

local ScriptEnv = {}
ScriptEnv.Easing = CoroutineUtility.EasingFunctions

local message_window = AdventureSystem:GetMessageWindow()
local message_window_open = false

local direction = AdventureSystem:GetDirection()
local sound = AdventureSystem:GetSound()
local stroke = AdventureSystem:GetStroke()

-- functions
-- Wait
function ScriptEnv.Wait(time)
	while time > 0.0 do
		time = time - coroutine.yield()
	end
end

-- message window
function ScriptEnv.LoadMessage(path, name)
	message_window:LoadMessage(path, name)
	while not message_window:is_message_load() do
		coroutine.yield()
	end
end
function ScriptEnv.SetMessage(id,face,stopFace)
	ScriptEnv.OpenWindow()
	message_window:SetMessage(id,face,stopFace)
	message_window:Play()
	while not message_window:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.SetFontSize(font_size)
	message_window:SetFontSize(font_size)
end
function ScriptEnv.ResetFontSize()
	message_window:ResetFontSize()
end
function ScriptEnv.SetShakeMessage(id,face,stopFace, power)
	ScriptEnv.OpenWindow()
	message_window:ShakeWindow(power)
	message_window:SetMessage(id,face,stopFace)
	message_window:Play()
	while not message_window:is_end() do
		coroutine.yield()
	end
end

function ScriptEnv.SetAnnouncementParam(position_x,position_y,position_z,scale,ID)
	message_window:SetAnnouncementParam(position_x,position_y,position_z,scale,ID)
end
function ScriptEnv.HideMessage()
	message_window:HideMessage()
end
function ScriptEnv.PlayAnnouncement(wait_time,firstImage,secondImage,thirdImage)
	message_window:PlayAnnouncement(wait_time,firstImage,secondImage,thirdImage)
	while not message_window:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.SetAnnouncement()
	message_window:SetAnnouncement()
end
function ScriptEnv.SetAutoMessage(id,face,stopFace,time,screen_skip)
	ScriptEnv.OpenWindow()
	message_window:SetMessage(id,face,stopFace,screen_skip,true,time)
	message_window:Play()
	while not message_window:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.SetAutoMessage(id,time,screen_skip)
	ScriptEnv.OpenWindow()
	message_window:SetMessage(id,0,0,screen_skip,true,time)
	message_window:Play()
	while not message_window:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.GetChoice()
	return message_window:GetChoice()
end
function ScriptEnv.SetChoice( begin, choice1, choice2 )
	stroke:ShowEventLogo(1)
	while not stroke:is_end() do
		coroutine.yield()
	end
	ScriptEnv.OpenWindow()
	message_window:SetMessage(begin,false,false)
	message_window:Play()
	ScriptEnv.Wait(3)
	message_window:SetChoice(choice1,choice2)
	while not message_window:is_end() do
		coroutine.yield()
	end
	ScriptEnv.CloseWindow()
	return message_window:GetChoice()
end
function ScriptEnv.EndChoice()
	stroke:HideEventLogo()
end
function ScriptEnv.SetStroke( chara, begin, half_begin, half_end, success )
	ScriptEnv.SetAutoMessage( begin, 2.3, true )
	ScriptEnv.CloseWindow()
	stroke:Start(chara)
	repeat
		while not stroke:is_end() do
			coroutine.yield()
		end
		local ret = stroke:result()
		if ret == 1 then
			stroke:Reset()
			local half_mes = math.random(half_begin, half_end)
			ScriptEnv.SetAutoMessage( half_mes, 0.8, false )
			ScriptEnv.CloseWindow()
			if stroke:is_end() == false then
				stroke:Restart()
			end
		end
	until ret == 2
	ScriptEnv.Wait(2)
	ScriptEnv.SetAutoMessage( success, 2.3, true )
	ScriptEnv.CloseWindow()
	stroke:End()
end

function ScriptEnv.ResetWindow()
	message_window:Reset()
	--[[
	while not message_window:is_end() do
		coroutine.yield()
	end
	]]
end
function ScriptEnv.OpenWindow()
	if not message_window_open then
		message_window_open = true
		ScriptEnv.ResetWindow()
		message_window:Open()
		while not message_window:is_end() do
			coroutine.yield()
		end
	end
end
function ScriptEnv.CloseWindow()
	if message_window_open then
		message_window_open = false
		message_window:Close()
		while not message_window:is_end() do
			coroutine.yield()
		end
	end
end

-- direction
function ScriptEnv.FadeIn(color,time)
	if color == nil then
		color = Color.New(1,1,1)
	end
	if time == nil then
		time = 0.15
	end
	direction:FadeIn(color,time)
	while not direction:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.FadeOut(color,time)
	if color == nil then
		color = Color.New(1,1,1,0)
	end
	if time == nil then
		time = 0.15
	end
	direction:FadeOut(color,time)
	while not direction:is_end() do
		coroutine.yield()
	end
end
function ScriptEnv.FadeInit(color)
	if color == nil then
		color = Color.New(1,1,1)
	end
	direction:SetFadeColor(color)
end
function ScriptEnv.Flash(color,count)
	if count == nil then
		count = 3
	end
	for i = 1, count do
		direction:Flash(color)
		while not direction:is_end() do
			coroutine.yield()
		end
	end
	direction:FlashEnd()
end
function ScriptEnv.ShakeCamera(frame,vec)
	direction:ShakeCamera(frame,vec)
	while not direction:is_end() do
		coroutine.yield()
	end
end

-- character
function ScriptEnv.LoadCharacter(id, position_x, position_y, position_z, rotation_x, rotation_y, rotation_z, scale_x, scale_y, scale_z)
	AdventureSystem:LoadCharacter(id, Vector3.New(position_x, position_y, position_z), Quaternion.Euler(rotation_x, rotation_y, rotation_z), Vector3.New(scale_x, scale_y, scale_z))
	while not AdventureSystem:CharacterIsLoaded() do
		coroutine.yield()
	end
	local chara = AdventureSystem:GetLastLoadCharacter()
	chara:PlayAnimation('idle')
	return chara
end
function ScriptEnv.LoadCharacter2(character_name, body_name)
	if body_name == nil then
		body_name = 'seifuku_00'
	end
	AdventureSystem:LoadCharacter2(character_name, body_name)
	while not AdventureSystem:CharacterIsLoaded() do
		coroutine.yield()
	end
	local chara = AdventureSystem:GetLastLoadCharacter()
	chara:PlayAnimation('idle')
	return chara
end
function ScriptEnv.GetRegistCharacter(id)
	return AdventureSystem:GetRegistCharacter(id)
end

-- BG
function ScriptEnv.LoadBg(file_name)
	AdventureSystem:LoadBg(file_name)
	while not AdventureSystem:BgIsLoaded() do
		coroutine.yield()
	end
	return AdventureSystem:GetLastLoadBg()
end

-- camera3D
function ScriptEnv.GetCamera3D()
	return AdventureSystem:GetCamera3D()
end

-- thread
function ScriptEnv.CreateThread(func)
	return ThreadList.New(LocalScriptEnv().thread_list_:CreateThread(func))
end
function ScriptEnv.WaitThread(thread)
	if thread == nil then
		thread = LocalScriptEnv().thread_list_
	end

	while not thread:IsCompleted() do
		coroutine.yield()
	end
end

-- disposable objects
local function DisposeObject(self, ...)
	if self.this_object_ ~= nil then
		self.this_object_[self.dispose_function_name_](self.this_object_, ...)
		self.this_object_ = nil
	end
end
local function UnregisterAndDisposeObject(self, ...)
	ScriptEnv.UnregisterDisposableObject(self)
	DisposeObject(self, ...)
end
function ScriptEnv.RegisterDisposableObject(obj, function_name)
	local new_disposable_object = {
		this_object_ = obj,
		dispose_function_name_ = function_name
	}
	new_disposable_object[function_name] = UnregisterAndDisposeObject
	setmetatable(new_disposable_object, {__index = obj})
	table.insert(LocalScriptEnv().disposable_object_list_, new_disposable_object)
	return new_disposable_object
end
function ScriptEnv.UnregisterDisposableObject(obj)
	local list = LocalScriptEnv().disposable_object_list_
	for i, value in ipairs(list) do
		if value == obj then
			table.remove(list, i)
			break
		end
	end
end
function ScriptEnv.DisposeAllRegistedObject()
	local list = LocalScriptEnv().disposable_object_list_
	for i, value in ipairs(list) do
		DisposeObject(value)
	end
	LocalScriptEnv().disposable_object_list_ = {}
end

-- audio
function ScriptEnv.LoadACB(sheet_name)
	sound:LoadACB(sheet_name)
end
function ScriptEnv.LoadVoiceACB(sheet_name)
	sound:LoadVoiceACB(sheet_name)
end
function ScriptEnv.UnloadACB(sheet_name)
	sound:UnloadACB(sheet_name)
end
function ScriptEnv.PlayBGM(sheet_name,cue_name)
	sound:PlayBGM(sheet_name,cue_name)
end
function ScriptEnv.StopBGM()
	sound:StopBGM()
end
function ScriptEnv.PlaySE(sheet_name,cue_name)
	sound:PlaySE(sheet_name,cue_name)
end
function ScriptEnv.PlayVoice(sheet_name,cue_name)
	sound:PlayVoice(sheet_name,cue_name)
end

-- effect
function ScriptEnv.CreateEffect(name)
	return nil
end

-- battle
function ScriptEnv.CreateBattle()
	local adventure_battle = AdventureSystem:CreateBattle()
	adventure_battle:Initialize()
	while not adventure_battle:initialized() do
		coroutine.yield()
	end
	return ScriptEnv.RegisterDisposableObject(adventure_battle, 'Destroy')
end
-- animation clip
function ScriptEnv.LoadAnimationClip(name)
	local animation_clip = AdventureSystem:CreateAnimationClip()
	animation_clip:Load(name)
	while animation_clip:loading() do
		coroutine.yield()
	end
	return animation_clip
end

-- metatable
setmetatable(ScriptEnv, { __index = _G })

return ScriptEnv
