-- Root.lua

Battle = Battle or {}
Battle.System = AdventureBattle.System

math.random = function(...) return Battle.System:LuaMathRandom(...) end

function RandomSelect(...)
	local length = select('#', ...)
	if length > 0 then
		return (select(math.random(length), ...))
	end
	return nil
end

-- CoroutineUtility
CoroutineUtility = dofile('Common/CoroutineUtility.lua')

-- TransformUtility
local TransformUtility = dofile('Common/TransformUtility.lua')
local TransformUtility_metatable = { __index = TransformUtility }

-- NilObject
NilObject = dofile('Common/NilObject.lua')

-- Character
do
	local Character = dofile('Battle/Character.lua')
	setmetatable(Battle.Character, { __index = Character })
	setmetatable(Character, TransformUtility_metatable)

	local original_play_voice = Battle.Character.PlayVoice
	function Battle.Character:PlayVoice(name)
		if not (Battle.System:skip() or self.dont_play_voice) then
			original_play_voice(self, name)
		end
	end
end

-- Effect
setmetatable(Battle.Effect, TransformUtility_metatable)

-- Model
do
	local Model = dofile('Battle/Model.lua')
	setmetatable(Battle.Model, { __index = Model })
	setmetatable(Model, TransformUtility_metatable)
end

-- ThreadList
ThreadList = dofile('Common/ThreadList.lua')

-- local env
Battle.local_script_env_ = nil

-- __newindex function
local function newindex(t, key, value)
	error('Can\'t set global table value in script. You tried to set a \''..key..'\'.', 2)
end

-- script env
Battle.script_env_ = dofile('AdventureBattle/ScriptEnv.lua')
do
	local metatable = {}
	metatable.__index = Battle.script_env_
	metatable.__newindex = newindex
	Battle.script_env_metatable_ = metatable
end

-- functions
function Battle.NewFunctionEnv(additional_env)
	local env = additional_env or {}
	env.thread_list_ = ThreadList.New()
	env.disposable_object_list_ = {}
	env.dispose_ = function()
		Battle.local_script_env_ = env
		Battle.script_env_.DisposeAllRegistedObject()
		Battle.local_script_env_ = nil
		setmetatable(env, nil)
		env.dispose_ = nil
	end
	setmetatable(env, Battle.script_env_metatable_)
	return env
end

function Battle.ScriptCoroutine(script, script_env, ...)
	local thread = coroutine.create(script)
	coroutine.yield()

	Battle.local_script_env_ = script_env
	local resume_result = { coroutine.resume(thread, ...) }
	local elapsed_time = 0.0
	while coroutine.status(thread) ~= 'dead' do
		script_env.thread_list_:Resume(elapsed_time)
		elapsed_time = coroutine.yield()
		Battle.local_script_env_ = script_env
		resume_result = { coroutine.resume(thread, elapsed_time) }
	end

	if resume_result[1] == false then
		ThreadList.CoroutineError(thread, resume_result[2])
	end
end
