-- CoroutineUtility.lua

local CoroutineUtility = {}

function CoroutineUtility.FixedTimeCoroutine(func, time, easing_func)
	if time == nil then
		return
	end
	if easing_func == nil then
		easing_func = CoroutineUtility.EasingFunctions.Linear
	end

	local timer = 0.0
	while timer < time do
		func(easing_func(timer / time))
		timer = timer + coroutine.yield()
	end
	func(1.0)
end

CoroutineUtility.EasingFunctions = {}
local Easing = CoroutineUtility.EasingFunctions

function Easing.Linear(t)
	return t
end
function Easing.InQuad(t)
	return t * t
end
function Easing.OutQuad(t)
	t = 1.0 - t
	return 1.0 - t * t
end
function Easing.InOutQuad(t)
	if t < 0.5 then
		return t * t * 2.0
	else
		t = 1.0 - t
		return 1.0 - t * t * 2.0
	end
end
function Easing.InCubic(t)
	return t * t * t
end
function Easing.OutCubic(t)
	t = 1.0 - t
	return 1.0 - t * t * t
end
function Easing.InOutCubic(t)
	if t < 0.5 then
		return t * t * t * 4.0
	else
		t = 1.0 - t
		return 1.0 - t * t * t * 4.0
	end
end
function Easing.InQuart(t)
	return t * t * t * t
end
function Easing.OutQuart(t)
	t = 1.0 - t
	return 1.0 - t * t * t * t
end
function Easing.InOutQuart(t)
	if t < 0.5 then
		return t * t * t * t * 8.0
	else
		t = 1.0 - t
		return 1.0 - t * t * t * t * 8.0
	end
end
function Easing.InQuint(t)
	return t * t * t * t * t
end
function Easing.OutQuint(t)
	t = 1.0 - t
	return 1.0 - t * t * t * t * t
end
function Easing.InOutQuint(t)
	if t < 0.5 then
		return t * t * t * t * t * 16.0
	else
		t = 1.0 - t
		return 1.0 - t * t * t * t * t * 16.0
	end
end
function Easing.InSine(t)
	return 1.0 - math.cos(t * math.pi * 0.5)
end
function Easing.OutSine(t)
	return math.sin(t * math.pi * 0.5)
end
function Easing.InOutSine(t)
	return (1.0 - math.cos(t * math.pi)) * 0.5
end
function Easing.InExpo(t)
	return math.pow(2, (t - 1.0) * 10)
end
function Easing.OutExpo(t)
	return 1.0 - math.pow(2, t * -10.0)
end
function Easing.InOutExpo(t)
	if t < 0.5 then
		return math.pow(2, (t * 2.0 - 1.0) * 10) * 0.5
	else
		return 1.0 - math.pow(2, (t - 0.5) * -20.0) * 0.5
	end
end
function Easing.InCirc(t)
	return 1.0 - math.sqrt(1.0 - t * t)
end
function Easing.OutCirc(t)
	t = 1.0 - t
	return math.sqrt(1.0 - t * t)
end
function Easing.InOutCirc(t)
	if t < 0.5 then
		return (1.0 - math.sqrt(1.0 - t * t * 4.0)) * 0.5
	else
		t = 1.0 - t
		return 0.5 + math.sqrt(1.0 - t * t * 4.0) * 0.5
	end
end
local function InBack(t, s)
	return t * t * (t * (s + 1.0) - s)
end
function Easing.InBack(t)
	return InBack(t, 1.70158)
end
function Easing.InBackWithParameter(s)
	return function(t) return InBack(t, s) end
end
local function OutBack(t, s)
	t = 1.0 - t
	return 1.0 - t * t * (t * (s + 1.0) - s)
end
function Easing.OutBack(t)
	return OutBack(t, 1.70158)
end
function Easing.OutBackWithParameter(s)
	return function(t) return OutBack(t, s) end
end
local function InOutBack(t, s)
	t = t * 2.0
	s = s * 1.525
	if t < 1.0 then
		return t * t * (t * (s + 1.0) - s) * 0.5
	else
		t = 2.0 - t
		return 1.0 - t * t * (t * (s + 1.0) - s) * 0.5
	end
end
function Easing.InOutBack(t)
	return InOutBack(t, 1.70158)
end
function Easing.InOutBackWithParameter(s)
	return function(t) return InOutBack(t, s) end
end
function Easing.InElastic(t)
	local p = 0.3
	local s = p * 0.25
	t = t - 1.0
	return -(math.pow(2, t * 10.0) * math.sin((t - s) * 2.0 * math.pi / p))
end
function Easing.OutElastic(t)
	local p = 0.3
	local s = p * 0.25
	return 1.0 + math.pow(2, t * -10.0) * math.sin((t - s) * 2.0 * math.pi / p)
end
function Easing.InOutElastic(t)
	local p = 0.3 * 1.5
	local s = p * 0.25
	t = t * 2.0 - 1.0
	if t < 0.0 then
		return -(math.pow(2, t * 10.0) * math.sin((t - s) * 2.0 * math.pi / p)) * 0.5
	else
		return 1.0 + math.pow(2, t * -10.0) * math.sin((t - s) * 2.0 * math.pi / p) * 0.5
	end
end
function Easing.InBounce(t)
	return 1.0 - Easing.OutBounce(1.0 - t)
end
function Easing.OutBounce(t)
	if t < 1.0 / 2.75 then
		return t * t * 7.5625
	elseif t < 2.0 / 2.75 then
		t = t - (1.5 / 2.75)
		return t * t * 7.5625 + 0.75
	elseif t < 2.5 / 2.75 then
		t = t - (2.25 / 2.75)
		return t * t * 7.5625 + 0.9375
	else
		t = t - (2.625 / 2.75)
		return t * t * 7.5625 + 0.984375
	end
end
function Easing.InOutBounce(t)
	t = t * 2.0
	if t < 1.0 then
		return 0.5 - Easing.OutBounce(1.0 - t) * 0.5
	else
		return 0.5 + Easing.OutBounce(t - 1.0) * 0.5
	end
end

return CoroutineUtility
