-- ThreadList.lua

local ThreadList = {}
local ThreadList_metatable = { __index = ThreadList }

function ThreadList.New(...)
	local instance = {...}
	setmetatable(instance, ThreadList_metatable)
	return instance
end

function ThreadList.CoroutineError(thread, error_message)
	local index = error_message:find(':', 1, true)
	if index ~= nil then
		index = error_message:find(':', index + 1, true)
		if index ~= nil then
			error_message = error_message:sub(index + 2)
		end
	end
	error(debug.traceback(thread, error_message, 0), 0)
end

function ThreadList:Resume(elapsed_time)
	local i = 1
	while i <= #self do
		local thread = self[i]
		local resume_result = { coroutine.resume(thread, elapsed_time) }
		if coroutine.status(thread) == 'dead' then
			if resume_result[1] == false then
				ThreadList.CoroutineError(thread, resume_result[2])
			end
			table.remove(self, i)
		else
			i = i + 1
		end
	end
end

function ThreadList:CreateThread(func)
	local thread = coroutine.create(func)
	table.insert(self, thread)
	return thread
end

function ThreadList:Clone()
	local list = ThreadList.New()
	table.copy(list, self)
	return list
end

function ThreadList:IsCompleted()
	for i, value in ipairs(self) do
		if coroutine.status(value) ~= 'dead' then
			return false
		end
	end
	return true
end

-- metatable
function ThreadList_metatable.__add(lhs, rhs)
	local list = lhs:Clone()
	table.copy(list, rhs)
	return list
end

return ThreadList
