-- TransformUtility.lua

local FixedTimeCoroutine = CoroutineUtility.FixedTimeCoroutine

local TransformUtility = {}

function TransformUtility:Move(distance, time, easing_func)
	local position = self:position()
	FixedTimeCoroutine(
		function(t)
			self:position(position + distance * t)
		end,
		time, easing_func)
	self:position(position + distance)
end
--[[
function TransformUtility:MoveSpeed(speed, time, easing_func)
	self:Move(speed * time, time, easing_func)
end
]]

function TransformUtility:MoveTo(to, time, easing_func)
	local move = to - self:position()
	self:Move(move, time, easing_func)
end
--[[
function TransformUtility:MoveToSpeed(to, speed, easing_func)
	local distance = to - self:position()
	self:Move(distance, distance:Magnitude() / speed, easing_func)
end
]]

function TransformUtility:MoveToQuadraticBezier(control_point, to, time, easing_func)
	local from = self:position()
	FixedTimeCoroutine(
		function(t)
			self:position(Vector3.QuadraticBezier(from, control_point, to, t))
		end,
		time, easing_func)
	self:position(to)
end
function TransformUtility:MoveToCubicBezier(control_point1, control_point2, to, time, easing_func)
	local from = self:position()
	FixedTimeCoroutine(
		function(t)
			self:position(Vector3.CubicBezier(from, control_point1, control_point2, to, t))
		end,
		time, easing_func)
	self:position(to)
end

function TransformUtility:Rotate(angle, time, easing_func)
	local from = self:rotation():eulerAngles()
	FixedTimeCoroutine(
		function(t)
			self:rotation(Quaternion.Euler(from + angle * t))
		end,
		time, easing_func)
	self:rotation(Quaternion.Euler(from + angle))
end

function TransformUtility:ScaleTo(to, time, easing_func)
	local from = self:scale()
	local distance = to - from
	FixedTimeCoroutine(
		function(t)
			self:scale(from + distance * t)
		end,
		time, easing_func)
	self:scale(to)
end

return TransformUtility