-- ScriptEnv.lua

local function LocalScriptEnv()
	return HomeMenu.local_script_env_
end

local ScriptEnv = {}
ScriptEnv.Easing = CoroutineUtility.EasingFunctions


-- functions
-- Wait
function ScriptEnv.Wait(time)
	while time > 0.0 do
		time = time - coroutine.yield()
	end
end

-- thread
function ScriptEnv.CreateThread(func)
	return ThreadList.New(LocalScriptEnv().thread_list_:CreateThread(func))
end
function ScriptEnv.WaitThread(thread)
	if thread == nil then
		thread = LocalScriptEnv().thread_list_
	end

	while not thread:IsCompleted() do
		coroutine.yield()
	end
end

-- disposable objects
local function DisposeObject(self, ...)
	if self.this_object_ ~= nil then
		self.this_object_[self.dispose_function_name_](self.this_object_, ...)
		self.this_object_ = nil
	end
end
local function UnregisterAndDisposeObject(self, ...)
	ScriptEnv.UnregisterDisposableObject(self)
	DisposeObject(self, ...)
end
function ScriptEnv.RegisterDisposableObject(obj, function_name)
	local new_disposable_object = {
		this_object_ = obj,
		dispose_function_name_ = function_name
	}
	new_disposable_object[function_name] = UnregisterAndDisposeObject
	setmetatable(new_disposable_object, {__index = obj})
	table.insert(LocalScriptEnv().disposable_object_list_, new_disposable_object)
	return new_disposable_object
end
function ScriptEnv.UnregisterDisposableObject(obj)
	local list = LocalScriptEnv().disposable_object_list_
	for i, value in ipairs(list) do
		if value == obj then
			table.remove(list, i)
			break
		end
	end
end
function ScriptEnv.DisposeAllRegistedObject()
	local list = LocalScriptEnv().disposable_object_list_
	for i, value in ipairs(list) do
		DisposeObject(value)
	end
	LocalScriptEnv().disposable_object_list_ = {}
end

-- metatable
setmetatable(ScriptEnv, { __index = _G })

return ScriptEnv
