-- table

-- copy
function table.copy(dest, src)
	for key, value in pairs(src) do
		dest[key] = value
	end
	return dest
end

-- clone
function table.clone(t)
	return table.copy({}, t)
end

-- raw_copy
function table.raw_copy(dest, src)
	for key, value in pairs(src) do
		local raw_value = rawget(src, key)
		if raw_value == value then
			rawset(dest, key, raw_value)
		end
	end
	return dest
end

--[[
-- raw_deep_copy
function table.raw_deep_copy(dest, src)
	table.raw_copy(dest, src)

	local src_metatable = getmetatable(src)
	if src_metatable ~= nil then
		local dest_metatable = getmetatable(dest)
		if dest_metatable == nil then
			dest_metatable = {}
			setmetatable(dest, dest_metatable)
		end
		table.raw_deep_copy(dest_metatable, src_metatable)
	end

	return dest
end
]]

-- raw_clone
function table.raw_clone(t)
	return table.raw_copy({}, t)
end


-- dump
do
	local exist_table = function(table_list, target)
		for i, value in ipairs(table_list) do
			if value == target then
				return true
			end
		end
		return false
	end
	local func = nil
	func = function(target, depth, path, table_list)
		table.insert(table_list, target)
		for key, value in pairs(target) do
			local value_path
			if type(key) == 'number' then
				value_path = path..'['..key..']'
			else
				value_path = path..'.'..key
			end
			if depth == 0 or rawget(target, key) ~= nil then
				print(value_path..'=', value)
			end
			if depth ~= 0 and type(value) == 'table' then
				if exist_table(table_list, value) == false then
					func(value, depth - 1, value_path, table_list)
				end
			end
		end
		table.remove(table_list)
	end
	table.dump = function(target, depth, path)
		if type(target) ~= 'table' then
			print('table.dump() error: first argument must be table type.')
			return
		end
		if depth == nil then
			depth = 0
		end
		if path == nil then
			path = ''
		end
		local table_list = {}
		func(target, depth, path, table_list)
	end
end
