-- Root.lua

-- override lua functions
-- print
if where ~= nil then
	local original_print = print
	function print(...)
		original_print(where(2), ...)
	end
end
-- print_warning
if where ~= nil then
	local original_print_warning = print_warning
	function print_warning(...)
		original_print_warning(where(2), ...)
	end
end
-- print_error
if where ~= nil then
	local original_print_error = print_error
	function print_error(...)
		original_print_error(where(2), ...)
	end
end


-- loadfile
function loadfile(filename, mode, env)
	local path = UnityClasses.WWW.GetStreamingAssetsPath('LuaRoot/'..filename)
	local www = UnityClasses.WWW.New(path)
	while not www:isDone() do
		coroutine.yield()
	end
	if www:isError() then
		local message = www:error()
		www:Dispose()
		return nil, message
	end
	local func, message = www:ToLuaFunction(mode, env)
	www:Dispose()
	return func, message
end
--- dofile
function dofile(path, ...)
	local func, message = loadfile(path)
	if func == nil then
		error(message)
	end
	return func(...)
end

-- table
dofile('LuaExtension/Table.lua')

-- coroutine
do
	local original_yield = coroutine.yield
	local function check_result(result, ...)
		if result then
			return ...
		end
		error('attempt to yield from outside a coroutine', 2)
	end
	coroutine.yield = function(...)
		return check_result(pcall(original_yield, ...))
	end
end

-- Utility
Utility = dofile('Common/Utility.lua')

-- UnityClasses
-- Vector2
Vector2 = dofile('UnityClasses/Vector2.lua')
-- Vector3
Vector3 = dofile('UnityClasses/Vector3.lua')
-- Vector4
Vector4 = dofile('UnityClasses/Vector4.lua')
-- Quaternion
Quaternion = dofile('UnityClasses/Quaternion.lua')
-- Color
Color = dofile('UnityClasses/Color.lua')

-- CSharp
-- Reflection
--Reflection = dofile('CSharp/Reflection.lua')
