-- Color.lua

local Color = {}
local Color_metatable = {}
setmetatable(Color, Color_metatable)

-- static
function Color.New(...)
	local arguments = {...}
	if #arguments <= 0 then
		return { r = 0.0, g = 0.0, b = 0.0, a = 0.0 }
	end
	return {
		r = arguments[1] or 0.0,
		g = arguments[2] or 0.0,
		b = arguments[3] or 0.0,
		a = arguments[4] or 1.0
	}
end

function Color.White()
	return Color.New(1.0, 1.0, 1.0)
end

function Color.Black()
	return Color.New(0.0, 0.0, 0.0)
end

function Color.Lerp(c1, c2, t)
	return Color.New(
		c1.r + (c2.r - c1.r) * t,
		c1.g + (c2.g - c1.g) * t,
		c1.b + (c2.b - c1.b) * t,
		c1.a + (c2.a - c1.a) * t)
end

-- metatable
function Color_metatable.__call(t, ...)
	return Color.New(...)
end

return Color
