-- Quaternion.lua

local Quaternion = {}
local Quaternion_metatable = { __index = UnityClasses.Quaternion }
setmetatable(Quaternion, Quaternion_metatable)
local Quaternion_instance_functions = {}
local Quaternion_instance_metatable = { __index = Quaternion_instance_functions }


-- static
function Quaternion.New(x, y, z, w)
	local instance = {
		x = x or 0.0,
		y = y or 0.0,
		z = z or 0.0,
		w = w or 1.0
	}
	setmetatable(instance, Quaternion_instance_metatable)
	return instance
end

function Quaternion.IsInherited(t)
	return Utility.IsInherited(t, Quaternion_instance_metatable)
end

function Quaternion.identity()
	return Quaternion.New(0.0, 0.0, 0.0, 1.0)
end

-- metatable
function Quaternion_metatable.__call(t, ...)
	return Quaternion.New(...)
end

-- instance
function Quaternion_instance_functions:Clone()
	return Quaternion.New(self.x, self.y, self.z, self.w)
end

function Quaternion_instance_functions:ToString()
	return '('..self.x..','..self.y..','..self.z..','..self.w..')'
end

function Quaternion_instance_functions:eulerAngles()
	return UnityClasses.Quaternion.ToEulerAngles(self)
end

-- metatable
function Quaternion_instance_metatable.__mul(lhs, rhs)
	if Quaternion.IsInherited(rhs) then
		return UnityClasses.Quaternion.MultiQuaternion(lhs, rhs)
	end
	if Vector3.IsInherited(rhs) then
		return UnityClasses.Quaternion.MultiVector3(lhs, rhs)
	end
	error('Quaternion.__mul error: not supported type parameter.')
end

return Quaternion
