-- Vector2.lua

local Vector2 = {}
local Vector2_metatable = {}
setmetatable(Vector2, Vector2_metatable)
local Vector2_instance_functions = {}
local Vector2_instance_metatable = { __index = Vector2_instance_functions }

-- static
function Vector2.New(x, y)
	local instance = {
		x = x or 0.0,
		y = y or 0.0,
	}
	setmetatable(instance, Vector2_instance_metatable)
	return instance
end

function Vector2.IsInherited(t)
	return Utility.IsInherited(t, Vector2_instance_metatable)
end

function Vector2.Dot(lhs, rhs)
	return lhs.x * rhs.x + lhs.y * rhs.y
end

function Vector2.zero()
	return Vector2.New(0.0, 0.0)
end

function Vector2.one()
	return Vector2.New(1.0, 1.0)
end

function Vector2.Scale(lhs, rhs)
	return Vector2.New(lhs.x * rhs.x, lhs.y * rhs.y)
end

function Vector2.Lerp(v1, v2, t)
	return Vector2.New(
		v1.x + (v2.x - v1.x) * t,
		v1.y + (v2.y - v1.y) * t)
end

-- metatable
function Vector2_metatable.__call(t, ...)
	return Vector2.New(...)
end

-- instance
function Vector2_instance_functions:Clone()
	return Vector2.New(self.x, self.y)
end

function Vector2_instance_functions:Normalize()
	local reverse_magnitude = 1.0 / self:Magnitude()
	self.x = self.x * reverse_magnitude
	self.y = self.y * reverse_magnitude
end

function Vector2_instance_functions:Normalized()
	local clone = self:Clone()
	clone:Normalize()
	return clone
end

function Vector2_instance_functions:Magnitude()
	return math.sqrt(self:SqrMagnitude())
end

function Vector2_instance_functions:SqrMagnitude()
	return self.x * self.x + self.y * self.y
end

function Vector2_instance_functions:ToString()
	return '('..self.x..','..self.y..')'
end

-- metatable
function Vector2_instance_metatable.__add(lhs, rhs)
	return Vector2.New(lhs.x + rhs.x, lhs.y + rhs.y)
end
function Vector2_instance_metatable.__sub(lhs, rhs)
	return Vector2.New(lhs.x - rhs.x, lhs.y - rhs.y)
end
function Vector2_instance_metatable.__mul(lhs, rhs)
	return Vector2.New(lhs.x * rhs, lhs.y * rhs)
end

return Vector2
